@@ -29,6 +29,8 @@
 #include <config.h>
 #endif
 
+#include <wchar.h>
+
 #include "port.h"
 #include "minicom.h"
 #include "intl.h"
@@ -52,10 +54,12 @@ static int mcd(char *dir)
     if (*dir == 0)
       return 0;
     init = 1;
-    getcwd(odir, 255);
+    if (getcwd(odir, sizeof(odir)) == NULL)
+      return -1;
   }
   if (*dir == 0) {
-    chdir(odir);
+    if (chdir(odir) == -1)
+      return -1;
     return 0;
   }
 
@@ -298,10 +302,13 @@ void updown(int what, int nr)
     snprintf(title, sizeof(title), _("%.30s %s - Press CTRL-C to quit"), P_PNAME(g),
              what == 'U' ? _("upload") : _("download"));
     mc_wtitle(win, TMID, title);
-    pipe(pipefd);
+    if (pipe(pipefd) == -1)
+      werror("pipe() call failed");
   } else
     mc_wleave();
 
+  m_flush(portfd);
+
   switch (udpid = fork()) {
     case -1:
       werror(_("Out of memory: could not fork()"));
@@ -441,33 +448,53 @@ void updown(int what, int nr)
 
 void lockfile_remove(void)
 {
-  if (lockfile[0])
-    unlink(lockfile);
-}
-
-void lockfile_create(void)
-{
-  int fd, n;
-
-  if (!lockfile[0])
+  if (portfd_is_socket)
     return;
 
+#if !HAVE_LOCKDEV
+  if (lockfile[0])
+    unlink(lockfile);
+#else
+  ttyunlock(dial_tty);
+#endif
+}
+
+int lockfile_create(void)
+{
+  int n;
+
+  if (portfd_is_socket)
+    return 0;
+
+#if !HAVE_LOCKDEV
+  if (!lockfile[0])
+    return 0;
+
+  int fd;
   n = umask(022);
   /* Create lockfile compatible with UUCP-1.2 */
   if ((fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666)) < 0) {
     werror(_("Cannot create lockfile!"));
   } else {
-    // previously used lockfile format:
-    // char buf[81];
-    //snprintf(buf, sizeof(buf),  "%05d minicom %.20s\n", (int)getpid(), username);
     // FHS format:
     char buf[12];
     snprintf(buf, sizeof(buf),  "%10d\n", getpid());
     buf[sizeof(buf) - 1] = 0;
-    write(fd, buf, strlen(buf));
+    if (write(fd, buf, strlen(buf)) < (ssize_t)strlen(buf))
+      fprintf(stderr, _("Failed to write lockfile %s\n"), lockfile);
     close(fd);
   }
   umask(n);
+  return 0;
+#else
+  n = ttylock(dial_tty);
+  if (n < 0) {
+    fprintf(stderr, _("Cannot create lockfile for %s: %s\n"), dial_tty, strerror(-n));
+  } else if (n > 0) {
+    fprintf(stderr, _("Device %s is locked.\n"), dial_tty);
+  }
+  return n;
+#endif
 }
 
 /*
@@ -680,7 +707,8 @@ void runscript(int ask, const char *s, const char *l, const char *p)
   }
   scriptname(scr_name);
 
-  pipe(pipefd);
+  if (pipe(pipefd) < 0)
+    return;
 
   if (mcd(P_SCRIPTDIR) < 0)
     return;
